@@ -6,6 +6,7 @@ from aerosandbox.geometry import Airfoil
 from typing import Union, List, Dict
 import tempfile
 import warnings
+import os
 
 
 class XFoil(ExplicitAnalysis):
@@ -39,6 +40,7 @@ class XFoil(ExplicitAnalysis):
                  n_crit: float = 9.,
                  xtr_upper: float = 1.,
                  xtr_lower: float = 1.,
+                 full_potential: bool = False,
                  max_iter: int = 100,
                  xfoil_command: str = "xfoil",
                  xfoil_repanel: bool = True,
@@ -47,7 +49,8 @@ class XFoil(ExplicitAnalysis):
                  working_directory: str = None,
                  ):
         """
-        Interface to XFoil.
+        Interface to XFoil. Compatible with both XFoil v6.xx (public) and XFoil v7.xx (private, contact Mark Drela at
+        MIT for a copy.)
 
         Args:
 
@@ -63,6 +66,12 @@ class XFoil(ExplicitAnalysis):
 
             xtr_lower: The lower-surface trip location [x/c]
 
+            full_potential: If this is set True, it will turn full-potential mode on. Note that full-potential mode
+            is only available in XFoil v7.xx or higher. (Unless you have specifically gone through the trouble of
+            acquiring a copy of XFoil v7.xx you likely have v6.xx. Version 7.xx is not publicly distributed as of
+            2022; contact Mark Drela at MIT for a copy.) Note that if you enable this flag with XFoil v6.xx,
+            you'll likely get an error (no output file generated).
+
             max_iter: How many iterations should we let XFoil do?
 
             xfoil_command: The command-line argument to call XFoil.
@@ -96,12 +105,16 @@ class XFoil(ExplicitAnalysis):
             somewhere local for debugging purposes.
 
         """
+        if mach >= 1:
+            raise ValueError("XFoil will terminate if a supersonic freestream Mach number is given.")
+
         self.airfoil = airfoil
         self.Re = Re
         self.mach = mach
         self.n_crit = n_crit
         self.xtr_upper = xtr_upper
         self.xtr_lower = xtr_lower
+        self.full_potential = full_potential
         self.max_iter = max_iter
         self.xfoil_command = xfoil_command
         self.xfoil_repanel = xfoil_repanel
@@ -109,9 +122,6 @@ class XFoil(ExplicitAnalysis):
         self.timeout = timeout
         self.working_directory = working_directory
 
-        if np.length(self.airfoil.coordinates) > 401:  # If the airfoil coordinates exceed Fortran array allocation
-            self.xfoil_repanel = True
-
     def _default_keystrokes(self) -> List[str]:
         run_file_contents = []
 
@@ -146,6 +156,14 @@ class XFoil(ExplicitAnalysis):
             f"m {self.mach}",
         ]
 
+        if self.full_potential:
+            run_file_contents += [
+                "full",
+                "fpar",
+                f"i {self.max_iter}",
+                "",
+            ]
+
         # Handle iterations
         run_file_contents += [
             f"iter {self.max_iter}",
@@ -207,6 +225,12 @@ class XFoil(ExplicitAnalysis):
                 "quit"
             ]
 
+            # Remove an old output file, if one exists:
+            try:
+                os.remove(directory / output_filename)
+            except FileNotFoundError:
+                pass
+
             ### Execute
             try:
                 subprocess.run(
@@ -214,42 +238,69 @@ class XFoil(ExplicitAnalysis):
                     input="\n".join(keystrokes),
                     cwd=directory,
                     stdout=None if self.verbose else subprocess.DEVNULL,
+                    stderr=None if self.verbose else subprocess.DEVNULL,
                     text=True,
-                    timeout=self.timeout
+                    shell=True,
+                    timeout=self.timeout,
+                    check=True
                 )
             except subprocess.TimeoutExpired:
-                pass
+                warnings.warn(
+                    "XFoil run timed out!\n"
+                    "If this was not expected, try increasing the `timeout` parameter\n"
+                    "when you create this AeroSandbox XFoil instance.",
+                    stacklevel=2
+                )
+            except subprocess.CalledProcessError as e:
+                if e.returncode == 11:
+                    raise RuntimeError(
+                        "XFoil segmentation-faulted. This is likely because your input airfoil has too many points.\n"
+                        "Try repaneling your airfoil with `Airfoil.repanel()` before passing it into XFoil.\n"
+                        "For further debugging, turn on the `verbose` flag when creating this AeroSandbox XFoil instance.")
+                elif e.returncode == 8 or e.returncode == 136:
+                    raise RuntimeError(
+                        "XFoil returned a floating point exception. This is probably because you are trying to start\n"
+                        "your analysis at an operating point where the viscous boundary layer can't be initialized based\n"
+                        "on the computed inviscid flow. (You're probably hitting a Goldstein singularity.) Try starting\n"
+                        "your XFoil run at a less-aggressive operating point.")
+                else:
+                    raise e
 
             ### Parse the polar
-            columns = [
-                "alpha",
-                "CL",
-                "CD",
-                "CDp",
-                "CM",
-                "xtr_upper",
-                "xtr_lower"
-            ]
+            try:
+                with open(directory / output_filename) as f:
+                    lines = f.readlines()
+            except FileNotFoundError:
+                raise FileNotFoundError(
+                    "It appears XFoil didn't produce an output file, probably because it crashed.\n"
+                    "Try running with `verbose=True` in the XFoil constructor to see what's going on."
+                )
 
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore")
-                try:
-                    output_data = np.genfromtxt(
-                        directory / output_filename,
-                        skip_header=12,
-                        usecols=np.arange(len(columns))
-                    ).reshape(-1, len(columns))
-                except OSError:  # File not found
-                    output_data = np.array([]).reshape(-1, len(columns))
+            title_line = lines[10]
+            columns = title_line.split()
 
-            has_valid_inputs = len(output_data) != 0
-
-            output_data_clean = {
-                k: output_data[:, index] if has_valid_inputs else np.array([])
-                for index, k in enumerate(columns)
+            output = {
+                column: []
+                for column in columns
             }
 
-            return output_data_clean
+            def str_to_float(s: str) -> float:
+                try:
+                    return float(s)
+                except ValueError:
+                    return np.nan
+
+            for line in lines[12:]:
+                data = [str_to_float(entry) for entry in line.split()]
+                for i in range(len(columns)):
+                    output[columns[i]].append(data[i])
+
+            output = {
+                k: np.array(v, dtype=float)
+                for k, v in output.items()
+            }
+
+            return output
 
     def alpha(self,
               alpha: Union[float, np.ndarray],
@@ -280,7 +331,7 @@ class XFoil(ExplicitAnalysis):
 
         if np.length(alphas) > 1:
             if start_at is not None:
-                if start_at > np.min(alphas) and start_at < np.max(alphas):
+                if np.min(alphas) < start_at < np.max(alphas):
                     alphas = np.sort(alphas)
                     alphas_upper = alphas[alphas > start_at]
                     alphas_lower = alphas[alpha <= start_at][::-1]
