@@ -5,10 +5,13 @@ from aerosandbox.geometry.airfoil.airfoil_families import get_NACA_coordinates,
     get_kulfan_coordinates, get_file_coordinates
 from aerosandbox.geometry.airfoil.default_airfoil_aerodynamics import default_CL_function, default_CD_function, \
     default_CM_function
+from aerosandbox.library.aerodynamics import transonic
+from aerosandbox.modeling.splines.hermite import linear_hermite_patch, cubic_hermite_patch
 from scipy import interpolate
 import matplotlib.pyplot as plt
 from typing import Callable, Union, Any, Dict
 import json
+from pathlib import Path
 
 
 class Airfoil(Polygon):
@@ -154,6 +157,9 @@ class Airfoil(Polygon):
                         cache_filename: str = None,
                         xfoil_kwargs: Dict[str, Any] = None,
                         unstructured_interpolated_model_kwargs: Dict[str, Any] = None,
+                        include_compressibility_effects: bool = True,
+                        transonic_buffet_lift_knockdown: float = 0.3,
+                        make_symmetric_polars: bool = False,
                         ) -> None:
         """
         Generates airfoil polars (CL, CD, CM functions) and assigns them in-place to this Airfoil's polar functions.
@@ -181,6 +187,10 @@ class Airfoil(Polygon):
             that contain the polars themselves. See the aerosandbox.UnstructuredInterpolatedModel constructor for
             options.
 
+            include_compressibility_effects: Includes compressibility effects in the polars, such as wave drag,
+            mach tuck, CL effects across normal shocks. Note that accuracy here is dubious in the transonic regime
+            and above - you should really specify your own CL/CD/CM models
+
         Warning: In-place operation! Modifies this Airfoil object by setting Airfoil.CL_function, etc. to the new
         polars.
 
@@ -257,6 +267,14 @@ class Airfoil(Polygon):
                 for k in run_datas[0].keys()
             }
 
+            if make_symmetric_polars:  # If the airfoil is known to be symmetric, duplicate all data across alpha.
+                keys_symmetric_across_alpha = ['CD', 'CDp', 'Re']
+
+                data = {
+                    k: np.concatenate([v, v if k in keys_symmetric_across_alpha else -v])
+                    for k, v in data.items()
+                }
+
             if cache_filename is not None:  # Cache the accumulated data for later use, if it doesn't already exist.
                 with open(cache_filename, "w+") as f:
                     json.dump(
@@ -357,12 +375,51 @@ class Airfoil(Polygon):
                 "ln_Re": np.log(Re),
             })
             CL_separated = CL_separated_interpolator(alpha)
-            return np.blend(
+
+            CL_mach_0 = np.blend(
                 separation_parameter(alpha, Re),
                 CL_separated,
                 CL_attached
             )
 
+            if include_compressibility_effects:
+                prandtl_glauert_beta_squared_ideal = 1 - mach ** 2
+
+                prandtl_glauert_beta = np.softmax(
+                    prandtl_glauert_beta_squared_ideal,
+                    -prandtl_glauert_beta_squared_ideal,
+                    hardness=2.0  # Empirically tuned to data
+                ) ** 0.5
+
+                CL = CL_mach_0 / prandtl_glauert_beta
+
+                mach_crit = transonic.mach_crit_Korn(
+                    CL=CL,
+                    t_over_c=self.max_thickness(),
+                    sweep=0,
+                    kappa_A=0.95
+                )
+
+                ### Accounts approximately for the lift drop due to buffet.
+                buffet_factor = np.blend(
+                    40 * (mach - mach_crit - (0.1 / 80) ** (1 / 3) - 0.06) * (mach - 1.1),
+                    1,
+                    transonic_buffet_lift_knockdown
+                )
+
+                ### Accounts for the fact that theoretical CL_alpha goes from 2 * pi (subsonic) to 4 (supersonic),
+                # following linearized supersonic flow on a thin airfoil.
+                cla_supersonic_ratio_factor = np.blend(
+                    10 * (mach - 1),
+                    4 / (2 * np.pi),
+                    1,
+                )
+
+                return CL * buffet_factor * cla_supersonic_ratio_factor
+
+            else:
+                return CL_mach_0
+
         def CD_function(alpha, Re, mach=0, deflection=0):
             alpha = np.mod(alpha + 180, 360) - 180  # Keep alpha in the valid range.
             log10_CD_attached = log10_CD_attached_interpolator({
@@ -370,12 +427,96 @@ class Airfoil(Polygon):
                 "ln_Re": np.log(Re),
             })
             log10_CD_separated = log10_CD_separated_interpolator(alpha)
-            return 10 ** np.blend(
+
+            log10_CD_mach_0 = np.blend(
                 separation_parameter(alpha, Re),
                 log10_CD_separated,
                 log10_CD_attached,
             )
 
+            if include_compressibility_effects:
+
+                CL_attached = CL_attached_interpolator({
+                    "alpha": alpha,
+                    "ln_Re": np.log(Re),
+                })
+                CL_separated = CL_separated_interpolator(alpha)
+
+                CL_mach_0 = np.blend(
+                    separation_parameter(alpha, Re),
+                    CL_separated,
+                    CL_attached
+                )
+                prandtl_glauert_beta_squared_ideal = 1 - mach ** 2
+
+                prandtl_glauert_beta = np.softmax(
+                    prandtl_glauert_beta_squared_ideal,
+                    -prandtl_glauert_beta_squared_ideal,
+                    hardness=2.0  # Empirically tuned to data
+                ) ** 0.5
+
+                CL = CL_mach_0 / prandtl_glauert_beta
+
+                t_over_c = self.max_thickness()
+
+                mach_crit = transonic.mach_crit_Korn(
+                    # CL=CL_function(alpha, Re, mach, deflection),
+                    CL=CL,
+                    t_over_c=t_over_c,
+                    sweep=0,
+                    kappa_A=0.92
+                )
+                mach_dd = mach_crit + (0.1 / 80) ** (1 / 3)
+                CD_wave = np.where(
+                    mach < mach_crit,
+                    0,
+                    np.where(
+                        mach < mach_dd,
+                        20 * (mach - mach_crit) ** 4,
+                        np.where(
+                            mach < 0.97,
+                            cubic_hermite_patch(
+                                mach,
+                                x_a=mach_dd,
+                                x_b=0.97,
+                                f_a=20 * (0.1 / 80) ** (4 / 3),
+                                f_b=0.8 * t_over_c,
+                                dfdx_a=0.1,
+                                dfdx_b=0.8 * t_over_c * 8
+                            ),
+                            np.where(
+                                mach < 1.1,
+                                cubic_hermite_patch(
+                                    mach,
+                                    x_a=0.97,
+                                    x_b=1.1,
+                                    f_a=0.8 * t_over_c,
+                                    f_b=0.8 * t_over_c,
+                                    dfdx_a=0.8 * t_over_c * 8,
+                                    dfdx_b=-0.8 * t_over_c * 8,
+                                ),
+                                np.blend(
+                                    8 * 2 * (mach - 1.1) / (1.2 - 0.8),
+                                    0.8 * 0.8 * t_over_c,
+                                    1.2 * 0.8 * t_over_c,
+                                )
+                            )
+                        )
+                    )
+                )
+
+                # CD_wave = transonic.approximate_CD_wave(
+                #     mach=mach,
+                #     mach_crit=mach_crit,
+                #     CD_wave_at_fully_supersonic=0.90 * self.max_thickness()
+                # )
+
+                return 10 ** log10_CD_mach_0 + CD_wave
+
+
+            else:
+                return 10 ** log10_CD_mach_0
+
         def CM_function(alpha, Re, mach=0, deflection=0):
             alpha = np.mod(alpha + 180, 360) - 180  # Keep alpha in the valid range.
             CM_attached = CM_attached_interpolator({
@@ -383,11 +524,26 @@ class Airfoil(Polygon):
                 "ln_Re": np.log(Re),
             })
             CM_separated = CM_separated_interpolator(alpha)
-            return np.blend(
+
+            CM_mach_0 = np.blend(
                 separation_parameter(alpha, Re),
                 CM_separated,
                 CM_attached
             )
+            if include_compressibility_effects:
+                prandtl_glauert_beta_squared_ideal = 1 - mach ** 2
+
+                prandtl_glauert_beta = np.softmax(
+                    prandtl_glauert_beta_squared_ideal,
+                    -prandtl_glauert_beta_squared_ideal,
+                    hardness=2.0  # Empirically tuned to data
+                ) ** 0.5
+
+                CM = CM_mach_0 / prandtl_glauert_beta
+
+                return CM
+            else:
+                return CM_mach_0
 
         self.CL_function = CL_function
         self.CD_function = CD_function
@@ -770,11 +926,72 @@ class Airfoil(Polygon):
             coordinates=coordinates
         )
 
+    def blend_with_another_airfoil(self,
+                                   airfoil: "Airfoil",
+                                   blend_fraction: float = 0.5,
+                                   n_points_per_side: int = 100,
+                                   ) -> "Airfoil":
+        """
+        Blends this airfoil with another airfoil. Merges both the coordinates and the aerodynamic functions.
+
+        Args:
+
+            airfoil: The other airfoil to blend with.
+
+            blend_fraction: The fraction of the other airfoil to use when blending. Defaults to 0.5 (50%).
+
+                * A blend fraction of 0 will return an identical airfoil to this one (self).
+
+                * A blend fraction of 1 will return an identical airfoil to the other one (`airfoil` parameter).
+
+            n_points_per_side: The number of points per side to use when blending the coordinates of the two airfoils.
+
+        Returns: A new airfoil that is a blend of this airfoil and another one.
+
+        """
+        this_foil = self.repanel(n_points_per_side=n_points_per_side)
+        that_foil = airfoil.repanel(n_points_per_side=n_points_per_side)
+        this_fraction = 1 - blend_fraction
+        that_fraction = blend_fraction
+
+        name = f"{this_fraction * 100:.0f}% {self.name}, {that_fraction * 100:.0f}% {airfoil.name}"
+
+        coordinates = (
+                this_fraction * this_foil.coordinates +
+                that_fraction * that_foil.coordinates
+        )
+
+        def CL_function(alpha, Re, mach, deflection):
+            return (
+                    this_fraction * this_foil.CL_function(alpha, Re, mach, deflection) +
+                    that_fraction * that_foil.CL_function(alpha, Re, mach, deflection)
+            )
+
+        def CD_function(alpha, Re, mach, deflection):
+            return (
+                    this_fraction * this_foil.CD_function(alpha, Re, mach, deflection) +
+                    that_fraction * that_foil.CD_function(alpha, Re, mach, deflection)
+            )
+
+        def CM_function(alpha, Re, mach, deflection):
+            return (
+                    this_fraction * this_foil.CM_function(alpha, Re, mach, deflection) +
+                    that_fraction * that_foil.CM_function(alpha, Re, mach, deflection)
+            )
+
+        return Airfoil(
+            name=name,
+            coordinates=coordinates,
+            CL_function=CL_function,
+            CD_function=CD_function,
+            CM_function=CM_function,
+        )
+
     # def normalize(self):
     #     pass  # TODO finish me
 
     def write_dat(self,
-                  filepath: str = None,
+                  filepath: Union[str, Path] = None,
                   include_name: bool = True,
                   ) -> str:
         """
@@ -937,7 +1154,7 @@ class Airfoil(Polygon):
     #     np.place(
     #         arr=xfoil_data_2D["Re"],
     #         mask=np.isnan(xfoil_data_2D["alpha"]),
-    #         vals=np.NaN
+    #         vals=np.nan
     #     )
     #     xfoil_data_2D["alpha_indices"] = np.arange(a_start, a_end + a_step / 2, a_step)
     #     xfoil_data_2D["Re_indices"] = Res
