@@ -1,6 +1,6 @@
 from aerosandbox import AeroSandboxObject
 from aerosandbox.geometry.common import *
-from typing import List, Dict, Any, Union
+from typing import List, Dict, Any, Union, Optional
 import aerosandbox.geometry.mesh_utilities as mesh_utils
 
 
@@ -16,14 +16,14 @@ class Airplane(AeroSandboxObject):
     """
 
     def __init__(self,
-                 name: str = "Untitled",
-                 xyz_ref: Union[np.ndarray, List] = np.array([0, 0, 0]),
-                 wings: List['Wing'] = None,
-                 fuselages: List['Fuselage'] = None,
-                 s_ref: float = None,
-                 c_ref: float = None,
-                 b_ref: float = None,
-                 analysis_specific_options: Dict[type, Dict[str, Any]] = None
+                 name: str = None,
+                 xyz_ref: Union[np.ndarray, List] = None,
+                 wings: Optional[List['Wing']] = None,
+                 fuselages: Optional[List['Fuselage']] = None,
+                 s_ref: Optional[float] = None,
+                 c_ref: Optional[float] = None,
+                 b_ref: Optional[float] = None,
+                 analysis_specific_options: Optional[Dict[type, Dict[str, Any]]] = None
                  ):
         """
         Defines a new airplane.
@@ -73,6 +73,10 @@ class Airplane(AeroSandboxObject):
 
         """
         ### Set defaults
+        if name is None:
+            name = "Untitled"
+        if xyz_ref is None:
+            xyz_ref = np.array([0., 0., 0.])
         if wings is None:
             wings: List['Wing'] = []
         if fuselages is None:
@@ -171,7 +175,7 @@ class Airplane(AeroSandboxObject):
              backend: str = "pyvista",
              thin_wings: bool = False,
              show: bool = True,
-             show_kwargs=None,
+             show_kwargs: Dict = None,
              ):
         """
 
@@ -213,7 +217,7 @@ class Airplane(AeroSandboxObject):
             return fig.draw(**show_kwargs)
         elif backend == "pyvista":
 
-            points, faces = self.mesh_body(method="quad")
+            points, faces = self.mesh_body(method="quad", thin_wings=thin_wings)
 
             import pyvista as pv
             fig = pv.PolyData(
@@ -229,7 +233,7 @@ class Airplane(AeroSandboxObject):
             return fig
         elif backend == "trimesh":
 
-            points, faces = self.mesh_body(method="tri")
+            points, faces = self.mesh_body(method="tri", thin_wings=thin_wings)
 
             import trimesh as tri
             fig = tri.Trimesh(points, faces)
