@@ -1,6 +1,6 @@
 from aerosandbox import AeroSandboxObject
 from aerosandbox.geometry.common import *
-from typing import List, Dict, Any, Tuple, Union
+from typing import List, Dict, Any, Tuple, Union, Optional
 from aerosandbox.geometry.airfoil import Airfoil
 from numpy import pi
 import aerosandbox.numpy as np
@@ -32,11 +32,11 @@ class Wing(AeroSandboxObject):
     """
 
     def __init__(self,
-                 name: str = "Untitled",
+                 name: str = None,
                  xsecs: List['WingXSec'] = None,
                  symmetric: bool = False,
-                 xyz_le: np.ndarray = None,  # Note: deprecated.
-                 analysis_specific_options: Dict[type, Dict[str, Any]] = None
+                 analysis_specific_options: Optional[Dict[type, Dict[str, Any]]] = None,
+                 **kwargs,  # Only to allow for capturing of deprecated arguments, don't use this.
                  ):
         """
         Defines a new wing.
@@ -74,6 +74,8 @@ class Wing(AeroSandboxObject):
                 >>> }
         """
         ### Set defaults
+        if name is None:
+            name = "Untitled"
         if xsecs is None:
             xsecs: List['WingXSec'] = []
         if analysis_specific_options is None:
@@ -86,7 +88,7 @@ class Wing(AeroSandboxObject):
         self.analysis_specific_options = analysis_specific_options
 
         ### Handle deprecated parameters
-        if xyz_le is not None:
+        if 'xyz_le' in locals():
             import warnings
             warnings.warn(
                 "The `xyz_le` input for Wing is DEPRECATED and will be removed in a future version. Use Wing().translate(xyz) instead.",
@@ -104,14 +106,14 @@ class Wing(AeroSandboxObject):
 
     def translate(self,
                   xyz: Union[np.ndarray, List]
-                  ):
+                  ) -> 'Wing':
         """
         Translates the entire Wing by a certain amount.
 
         Args:
             xyz:
 
-        Returns: self
+        Returns: The new wing object.
 
         """
         new_wing = copy.copy(self)
@@ -250,20 +252,20 @@ class Wing(AeroSandboxObject):
 
     def is_entirely_symmetric(self) -> bool:
         # Returns a boolean of whether the wing is totally symmetric (i.e.), every xsec has symmetric control surfaces.
-        for xsec in self.xsecs: # To be symmetric, all
+        for xsec in self.xsecs:  # To be symmetric, all
             for surf in xsec.control_surfaces:
                 if not (surf.symmetric or surf.deflection == 0):
                     return False
 
-        if not self.symmetric: # If the wing itself isn't mirrored (e.g., vertical stabilizer), check that it's symmetric
+        if not self.symmetric:  # If the wing itself isn't mirrored (e.g., vertical stabilizer), check that it's symmetric
             for xsec in self.xsecs:
-                if not xsec.xyz_le[1] == 0: # Surface has to be right on the centerline
+                if not xsec.xyz_le[1] == 0:  # Surface has to be right on the centerline
                     return False
-                if not xsec.twist == 0: # Surface has to be untwisted
+                if not xsec.twist == 0:  # Surface has to be untwisted
                     return False
-                if not xsec.airfoil.CL_function(0, 1e6, 0, 0) == 0: # Surface has to have a symmetric airfoil.
+                if not xsec.airfoil.CL_function(0, 1e6, 0, 0) == 0:  # Surface has to have a symmetric airfoil.
                     return False
-                if not xsec.airfoil.CM_function(0, 1e6, 0, 0) == 0: # Surface has to have a symmetric airfoil.
+                if not xsec.airfoil.CM_function(0, 1e6, 0, 0) == 0:  # Surface has to have a symmetric airfoil.
                     return False
 
         return True
@@ -536,8 +538,12 @@ class Wing(AeroSandboxObject):
         faces = np.array(faces)
 
         if self.symmetric:
-            flipped_points = np.array(points)
-            flipped_points[:, 1] = flipped_points[:, 1] * -1
+            flipped_points = np.multiply(
+                points,
+                np.array([
+                    [1, -1, 1]
+                ])
+            )
 
             points, faces = mesh_utils.stack_meshes(
                 (points, faces),
@@ -560,6 +566,27 @@ class Wing(AeroSandboxObject):
         Uses the `(points, faces)` standard mesh format. For reference on this format, see the documentation in
         `aerosandbox.geometry.mesh_utilities`.
 
+        Order of faces:
+            * On the right wing (or, if `Wing.symmetric` is `False`, just the wing itself):
+                * First face is the face nearest the leading edge of the wing root.
+                * Proceeds along a chordwise strip to the trailing edge.
+                * Then, goes to the subsequent spanwise location and does another chordwise strip, et cetera until
+                  we get to the wing tip.
+            * On the left wing (applicable only if `Wing.symmetric` is `True`):
+                * Same order: Starts at the root leading edge, goes in chordwise strips.
+
+        Order of vertices within each face:
+            * On the right wing (or, if `Wing.symmetric` is `False`, just the wing itself):
+                * Front-left
+                * Back-left
+                * Back-right
+                * Front-right
+            * On the left wing (applicable only if `Wing.symmetric` is `True`):
+                * Front-left
+                * Back-left
+                * Back-right
+                * Front-right
+
         Args:
             method: Allows choice between "tri" and "quad" meshing.
             chordwise_resolution: Controls the chordwise resolution of the meshing.
@@ -600,11 +627,11 @@ class Wing(AeroSandboxObject):
 
         faces = []
 
-        num_i = spanwise_resolution * (len(self.xsecs) - 1)
-        num_j = len(spanwise_strips) - 1
+        num_i = np.length(spanwise_strips[0])  # spanwise
+        num_j = np.length(spanwise_strips)  # chordwise
 
         def index_of(iloc, jloc):
-            return iloc + jloc * (num_i + 1)
+            return iloc + jloc * num_i
 
         def add_face(*indices):
             entry = list(indices)
@@ -614,25 +641,36 @@ class Wing(AeroSandboxObject):
                 faces.append([entry[0], entry[1], entry[3]])
                 faces.append([entry[1], entry[2], entry[3]])
 
-        for i in range(num_i):
-            for j in range(num_j):
-                add_face(
-                    index_of(i, j),
-                    index_of(i + 1, j),
-                    index_of(i + 1, j + 1),
-                    index_of(i, j + 1),
+        for i in range(num_i - 1):
+            for j in range(num_j - 1):
+                add_face(  # On right wing:
+                    index_of(i, j),  # Front-left
+                    index_of(i, j + 1),  # Back-left
+                    index_of(i + 1, j + 1),  # Back-right
+                    index_of(i + 1, j),  # Front-right
                 )
 
-        faces = np.array(faces)
-
         if self.symmetric:
-            flipped_points = np.array(points)
-            flipped_points[:, 1] = flipped_points[:, 1] * -1
+            index_offset = np.length(points)
 
-            points, faces = mesh_utils.stack_meshes(
-                (points, faces),
-                (flipped_points, faces)
-            )
+            points = np.concatenate([
+                points,
+                np.multiply(points, np.array([[1, -1, 1]]))
+            ])
+
+            def index_of(iloc, jloc):
+                return index_offset + iloc + jloc * num_i
+
+            for i in range(num_i - 1):
+                for j in range(num_j - 1):
+                    add_face(  # On left wing:
+                        index_of(i + 1, j),  # Front-left
+                        index_of(i + 1, j + 1),  # Back-left
+                        index_of(i, j + 1),  # Back-right
+                        index_of(i, j),  # Front-right
+                    )
+
+        faces = np.array(faces)
 
         return points, faces
 
@@ -665,7 +703,8 @@ class Wing(AeroSandboxObject):
 
         Returns:
 
-            points: a Nx3 np.ndarray that gives the coordinates of each point on the meshed line. Goes from the root to the tip.
+            points: a Nx3 np.ndarray that gives the coordinates of each point on the meshed line. Goes from the root
+            to the tip. Ignores any wing symmetry (e.g., only gives one side).
 
         """
 
@@ -718,13 +757,16 @@ class Wing(AeroSandboxObject):
 
         points_sections = []
         for i in range(len(xsec_points) - 1):
-            points_section = space(
-                xsec_points[i],
-                xsec_points[i + 1],
-                spanwise_resolution + 1
-            )
+            points_section = np.stack([
+                space(
+                    xsec_points[i][dim],
+                    xsec_points[i + 1][dim],
+                    spanwise_resolution + 1
+                )
+                for dim in range(3)
+            ], axis=1)
             if not i == len(xsec_points) - 2:
-                points_section = points_section[:-1]
+                points_section = points_section[:-1, :]
 
             points_sections.append(points_section)
 
@@ -746,6 +788,60 @@ class Wing(AeroSandboxObject):
         from aerosandbox.geometry.airplane import Airplane
         return Airplane(wings=[self]).draw(*args, **kwargs)
 
+    def subdivide_sections(self, ratio: int) -> "Wing":
+        """
+        Generates a new wing that subdivides the existing sections of this Wing into several smaller ones. Splits
+        each section into `ratio` smaller sub-sections by inserting new cross-sections (xsecs) as needed.
+
+        This can allow for finer aerodynamic resolution of sectional properties in certain analyses.
+
+        Args:
+            ratio: The number of new sections to split each old section into.
+
+        Returns: A new Wing object with subdivided sections.
+
+        """
+
+        new_xsecs = []
+        span_fractions_along_section = np.linspace(0, 1, ratio + 1)[:-1]
+
+        for xsec_a, xsec_b in zip(self.xsecs[:-1], self.xsecs[1:]):
+            for s in span_fractions_along_section:
+                a_weight = 1 - s
+                b_weight = s
+
+                if xsec_a.airfoil is xsec_b.airfoil:
+                    blended_airfoil = xsec_a.airfoil
+                elif a_weight == 1:
+                    blended_airfoil = xsec_a.airfoil
+                elif b_weight == 1:
+                    blended_airfoil = xsec_b.airfoil
+                else:
+                    blended_airfoil = xsec_a.airfoil.blend_with_another_airfoil(
+                        airfoil=xsec_b.airfoil,
+                        blend_fraction=b_weight
+                    )
+
+                new_xsecs.append(
+                    WingXSec(
+                        xyz_le=xsec_a.xyz_le * a_weight + xsec_b.xyz_le * b_weight,
+                        chord=xsec_a.chord * a_weight + xsec_b.chord * b_weight,
+                        twist=xsec_a.twist * a_weight + xsec_b.twist * b_weight,
+                        airfoil=blended_airfoil,
+                        control_surfaces=xsec_a.control_surfaces,
+                        analysis_specific_options=xsec_a.analysis_specific_options,
+                    )
+                )
+
+        new_xsecs.append(self.xsecs[-1])
+
+        return Wing(
+            name=self.name,
+            xsecs=new_xsecs,
+            symmetric=self.symmetric,
+            analysis_specific_options=self.analysis_specific_options
+        )
+
     def _compute_xyz_le_of_WingXSec(self, index: int):
         return self.xsecs[index].xyz_le
 
@@ -784,25 +880,30 @@ class Wing(AeroSandboxObject):
             the local reference frame of the WingXSec. Given in geometry axes.
 
         """
+
+        def project_to_YZ_plane_and_normalize(vector):
+            YZ_magnitude = (vector[1] ** 2 + vector[2] ** 2) ** 0.5
+            return np.array([0, vector[1], vector[2]]) / YZ_magnitude
+
         ### Compute the untwisted reference frame
         xg_local = np.array([1, 0, 0])
         if index == 0:
-            span_vector = self.xsecs[1].xyz_le - self.xsecs[0].xyz_le
-            span_vector[0] = 0
-            yg_local = span_vector / np.linalg.norm(span_vector)
+            yg_local = project_to_YZ_plane_and_normalize(
+                self.xsecs[1].xyz_le - self.xsecs[0].xyz_le
+            )
             z_scale = 1
         elif index == len(self.xsecs) - 1 or index == -1:
-            span_vector = self.xsecs[-1].xyz_le - self.xsecs[-2].xyz_le
-            span_vector[0] = 0
-            yg_local = span_vector / np.linalg.norm(span_vector)
+            yg_local = project_to_YZ_plane_and_normalize(
+                self.xsecs[-1].xyz_le - self.xsecs[-2].xyz_le
+            )
             z_scale = 1
         else:
-            vector_before = self.xsecs[index].xyz_le - self.xsecs[index - 1].xyz_le
-            vector_after = self.xsecs[index + 1].xyz_le - self.xsecs[index].xyz_le
-            vector_before[0] = 0  # Project onto YZ plane.
-            vector_after[0] = 0  # Project onto YZ plane.
-            vector_before = vector_before / np.linalg.norm(vector_before)
-            vector_after = vector_after / np.linalg.norm(vector_after)
+            vector_before = project_to_YZ_plane_and_normalize(
+                self.xsecs[index].xyz_le - self.xsecs[index - 1].xyz_le
+            )
+            vector_after = project_to_YZ_plane_and_normalize(
+                self.xsecs[index + 1].xyz_le - self.xsecs[index].xyz_le
+            )
             span_vector = (vector_before + vector_after) / 2
             yg_local = span_vector / np.linalg.norm(span_vector)
             cos_vectors = np.linalg.inner(vector_before, vector_after)
@@ -868,16 +969,13 @@ class WingXSec(AeroSandboxObject):
     """
 
     def __init__(self,
-                 xyz_le: Union[np.ndarray, List] = np.array([0, 0, 0]),
+                 xyz_le: Union[np.ndarray, List] = None,
                  chord: float = 1.,
-                 twist: float = 0,
+                 twist: float = 0.,
                  airfoil: Airfoil = None,
-                 control_surfaces: List['ControlSurface'] = None,
-                 analysis_specific_options: Dict[type, Dict[str, Any]] = None,
-                 control_surface_is_symmetric=None,  # Note: deprecated.
-                 control_surface_hinge_point=None,  # Note: deprecated.
-                 control_surface_deflection=None,  # Note: deprecated.
-                 twist_angle=None,  # Note: deprecated.
+                 control_surfaces: Optional[List['ControlSurface']] = None,
+                 analysis_specific_options: Optional[Dict[type, Dict[str, Any]]] = None,
+                 **deprecated_kwargs,
                  ):
         """
         Defines a new wing cross section.
@@ -968,6 +1066,8 @@ class WingXSec(AeroSandboxObject):
             See avl.py for example of control_surface handling using this protocol.
         """
         ### Set defaults
+        if xyz_le is None:
+            xyz_le = np.array([0., 0., 0.])
         if airfoil is None:
             airfoil = Airfoil("naca0012")
         if control_surfaces is None:
@@ -983,28 +1083,28 @@ class WingXSec(AeroSandboxObject):
         self.analysis_specific_options = analysis_specific_options
 
         ### Handle deprecated arguments
-        if twist_angle is not None:
+        if 'twist_angle' in deprecated_kwargs.keys():
             import warnings
             warnings.warn(
                 "DEPRECATED: 'twist_angle' has been renamed 'twist', and will break in future versions.",
                 stacklevel=2
             )
-            self.twist = twist_angle
-        if not (
-                control_surface_is_symmetric is None and
-                control_surface_hinge_point is None and
-                control_surface_deflection is None
+            self.twist = deprecated_kwargs['twist_angle']
+        if (
+                'control_surface_is_symmetric' in locals() or
+                'control_surface_hinge_point' in locals() or
+                'control_surface_deflection' in locals()
         ):
             import warnings
             warnings.warn(
                 "DEPRECATED: Define control surfaces using the `control_surfaces` parameter, which takes in a list of asb.ControlSurface objects.",
                 stacklevel=2
             )
-            if control_surface_is_symmetric is None:
+            if 'control_surface_is_symmetric' not in locals():
                 control_surface_is_symmetric = True
-            if control_surface_hinge_point is None:
+            if 'control_surface_hinge_point' not in locals():
                 control_surface_hinge_point = 0.75
-            if control_surface_deflection is None:
+            if 'control_surface_deflection' not in locals():
                 control_surface_deflection = 0
 
             self.control_surfaces.append(
@@ -1046,7 +1146,7 @@ class ControlSurface(AeroSandboxObject):
                  hinge_point: float = 0.75,
                  symmetric: bool = True,
                  deflection: float = 0.0,
-                 analysis_specific_options: Dict[type, Dict[str, Any]] = None,
+                 analysis_specific_options: Optional[Dict[type, Dict[str, Any]]] = None,
                  ):
         """
         Define a new control surface.
@@ -1109,13 +1209,13 @@ if __name__ == '__main__':
             WingXSec(
                 xyz_le=[0, 0, 0],
                 chord=1,
-                airfoil=Airfoil("naca0012"),
+                airfoil=Airfoil("naca4412"),
                 twist=0,
             ),
             WingXSec(
                 xyz_le=[0.5, 1, 0],
                 chord=0.5,
-                airfoil=Airfoil("naca0012"),
+                airfoil=Airfoil("naca4412"),
                 twist=0,
             ),
             WingXSec(
@@ -1126,4 +1226,4 @@ if __name__ == '__main__':
             )
         ]
     ).translate([1, 0, 0])
-    wing.draw()
+    wing.subdivide_sections(5).draw()
