@@ -13,6 +13,8 @@ def sears_haack_drag(
 
     https://en.wikipedia.org/wiki/Sears%E2%80%93Haack_body
 
+    Note that drag coefficient and drag area are independent of Mach number for this case (assuming linearized supersonic aero).
+
     Args:
         radius_max: The maximum radius of the Sears-Haack body.
         length: The length of the Sears-Haack body.
@@ -32,11 +34,39 @@ def sears_haack_drag_from_volume(
     See documentation for sears_haack_drag() in this same file.
 
     Identical, except takes volume as an input rather than max radius.
+
+    Also returns a drag area (denoted CDA, or equivalently, D/q).
     """
     CDA = 128 * volume ** 2 / (np.pi * length ** 4)
     return CDA
 
 
+def mach_crit_Korn(
+        CL,
+        t_over_c,
+        sweep=0,
+        kappa_A=0.95
+):
+    """
+        Wave drag_force coefficient prediction using the low-fidelity Korn Equation method;
+    derived in "Configuration Aerodynamics" by W.H. Mason, Sect. 7.5.2, pg. 7-18
+
+    Args:
+        CL: Sectional lift coefficient
+        t_over_c: thickness-to-chord ratio
+        sweep: sweep angle, in degrees
+        kappa_A: Airfoil technology factor (0.95 for supercritical section, 0.87 for NACA 6-series)
+
+    Returns:
+
+    """
+    smooth_abs_CL = np.softmax(CL, -CL, hardness=10)
+
+    M_dd = kappa_A / np.cosd(sweep) - t_over_c / np.cosd(sweep) ** 2 - smooth_abs_CL / (10 * np.cosd(sweep) ** 3)
+    M_crit = M_dd - (0.1 / 80) ** (1 / 3)
+    return M_crit
+
+
 def approximate_CD_wave(
         mach,
         mach_crit,
@@ -110,7 +140,7 @@ def approximate_CD_wave(
                     f_a=20 * (0.1 / 80) ** (4 / 3),
                     f_b=1,
                     dfdx_a=0.1,
-                    dfdx_b=10
+                    dfdx_b=8
                 ),
                 np.where(
                     mach < 1.2,
@@ -120,7 +150,7 @@ def approximate_CD_wave(
                         x_b=1.2,
                         f_a=1,
                         f_b=1,
-                        dfdx_a=10,
+                        dfdx_a=8,
                         dfdx_b=-4
                     ),
                     np.blend(
@@ -136,7 +166,7 @@ def approximate_CD_wave(
 
 
 if __name__ == '__main__':
-    mc = 0.7
+    mc = 0.6
     drag = lambda mach: approximate_CD_wave(
         mach,
         mach_crit=mc,
@@ -158,6 +188,7 @@ if __name__ == '__main__':
     plt.ylabel("$C_{D, wave} / C_{D, wave, M=1.2}$")
     plt.plot(mach, drag)
     plt.ylim(-0.05, 1.5)
+
     # plt.ylim(-0.01, 0.05)
 
     plt.sca(ax[1])
@@ -174,7 +205,7 @@ if __name__ == '__main__':
 
     for a in ax:
         plt.sca(a)
-        plt.xlim(0.5, 1.5)
+        plt.xlim(0.6, 1.2)
         plt.xlabel("Mach [-]")
 
     p.show_plot()
